// -*- c++ -*-

#ifndef LEMON_CLEARABLE_GRAPH_EXTENDER_H
#define LEMON_CLEARABLE_GRAPH_EXTENDER_H

#include <lemon/invalid.h>


namespace lemon {

  template <typename _Base> 
  class ClearableGraphExtender : public _Base {
  public:

    typedef ClearableGraphExtender Graph;
    typedef _Base Parent;
    typedef typename Parent::Node Node;
    typedef typename Parent::Edge Edge;

    void clear() {
      Parent::getNotifier(Node()).clear();
      Parent::getNotifier(Edge()).clear();
      Parent::clear();
    }

  };

  template <typename _Base> 
  class ClearableEdgeSetExtender : public _Base {
  public:

    typedef ClearableEdgeSetExtender Graph;
    typedef _Base Parent;
    typedef typename Parent::Node Node;
    typedef typename Parent::Edge Edge;

    void clear() {
      Parent::getNotifier(Edge()).clear();
      Parent::clear();
    }

  };

  template <typename _Base> 
  class ClearableUGraphExtender : public _Base {
  public:

    typedef ClearableUGraphExtender Graph;
    typedef _Base Parent;
    typedef typename Parent::Node Node;
    typedef typename Parent::UEdge UEdge;
    typedef typename Parent::Edge Edge;

    void clear() {
      Parent::getNotifier(Node()).clear();
      Parent::getNotifier(UEdge()).clear();
      Parent::getNotifier(Edge()).clear();
      Parent::clear();
    }
  };

  template <typename _Base> 
  class ClearableUEdgeSetExtender : public _Base {
  public:

    typedef ClearableUEdgeSetExtender Graph;
    typedef _Base Parent;
    typedef typename Parent::Node Node;
    typedef typename Parent::UEdge UEdge;
    typedef typename Parent::Edge Edge;

    void clear() {
      Parent::getNotifier(UEdge()).clear();
      Parent::getNotifier(Edge()).clear();
      Parent::clear();
    }

  };


  template <typename _Base>
  class ClearableBpUGraphExtender : public _Base {
  public:

    typedef _Base Parent;
    typedef ClearableBpUGraphExtender Graph;

    typedef typename Parent::Node Node;
    typedef typename Parent::BNode BNode;
    typedef typename Parent::ANode ANode;
    typedef typename Parent::Edge Edge;
    typedef typename Parent::UEdge UEdge;

    void clear() {
      Parent::getNotifier(Edge()).clear();
      Parent::getNotifier(UEdge()).clear();
      Parent::getNotifier(Node()).clear();
      Parent::getNotifier(BNode()).clear();
      Parent::getNotifier(ANode()).clear();
      Parent::clear();
    }

  };

}

#endif
